# Webcharts Reference

> Web components for plotting engineering and scientific data with a focus on performance.

**Table of Contents**

[Introduction](#introduction)
| [Using webcharts](#using-webcharts)
| [Styling webcharts](#styling-webcharts)
| [Webcharts API methods](#ni-cartesian-graph-base)
| [Graph](#ni-cartesian-graph)
| [Chart](#ni-chart)
| [Intensity Graph](#ni-intensity-graph)
| [Digital Graph](#ni-digital-graph)
| [Digital Axis](#ni-digital-axis)
| [Digital Signal](#ni-digital-signal)
| [Digital Bus](#ni-digital-bus)
| [Cartesian Axes](#ni-cartesian-axis)
| [Color Scale](#ni-color-scale)
| [Plots](#ni-cartesian-plot)
| [Plot renderer](#ni-cartesian-plot-renderer)
| [Cursors](#ni-cursor)
| [Axis Handle](#ni-axis-handle)
| [Parking Lot](#ni-parking-lot)
| [Typed WebCharts](#niTypedWebCharts)

## Introduction ##

Webcharts is a collection of [web components](https://www.webcomponents.org/introduction)
for plotting engineering and scientific data, with a focus on performance.

## Using webcharts ##

Webcharts is distributed through npm with the package name **ni-webcharts**.

Add webcharts to your app:

    npm install --save ni-webcharts

Add webcharts to a web page by including all script files listed in [exampleScripts.json](https://github.com/ni-kismet/webcharts/blob/master/exampleScripts.json).

## Styling webcharts ##

The way Webcharts look is configured using a mix of elements properties, described
below in this document, and CSS styles. A guide for styling with CSS is [here](styling.md).



## Webcharts API methods
The following methods are available for ni-cartesian-graph, ni-chart and
ni-intensity-graph.

**setData** is the prefered method to write data to a graph. The other
option, writing to the *value* property of the element is only recommended
for rarely changed small datasets, since it consumes more memory and CPU
due to the serialization/deserialization needed.

```js
// graph needs to be ready before calling any methods on it
var graph = document.querySelector('#myGraph');
graph.setData([1, 2, 3, 4, 5]);
```

**onRecenterPlot** is used to perform a reset for webcharts. This includes
resetting the offset of the axis and redrawing of the plot.

```js
// graph needs to be ready before calling any methods on it
var graph = document.querySelector('#myGraph');
graph.onRecenterPlot();
```

**updateCompleted** Promise is resolved when the graph has completed updating.
The graph element updates asynchronously in response to its and its nested elements' property changes.
Property changes are batched. If more properties change after an update is requested but before the update begins,
all of the changes are captured in the same update. This results in significant performance improvements in the rich
interaction scenario.

```js
var graph = document.querySelector('#myGraph');
graph.disablePan = true;
graph.disabled = true;
// The above two changes are batched
await graph.updateCompleted;
```


## Webcharts properties

**graph-ref** - a graph id used to bind legends to the graph

**plot-area-margin** - TODO - document this

**disable-pan** - set true to disable the mouse/touch drag interaction
to move the plot. Default: true.

**disable-zoom** - set true to disable the mouse wheel or touch pinch interaction
to zoom in or out the plot. Default: true.

**disable-recenter** - set true to disable the double click/tap interaction
to move and zoom back the plot. Default: true.

**pan-mode** - a string specifies the pan mode for mouse interaction. Accepted values:
'manual': no pan hint or direction snapping;
'smart': The graph shows pan hint bar and the pan movement will snap
to one direction when the drag direction is close to it;
'smartLock'. The graph shows pan hint bar and the pan movement will always
snap to a direction that the drag direction started with.
Default: 'smart'.

**touch-pan-mode** a string specifies the pan mode of touch pan.
The accepted values is the sams as **mode** option. Default: 'manual'

**propagate-supported-gesture** set true to allow the propagation of origin touch events
(e.g. 'touchstart') that is already handled for pan or pinch. Default: false.

**metadata-overrides-plot-names** (boolean) - true if channel names in waveform metadata should override names configured on `ni-cartesian-plot`s. Default: 'false'

## Webcharts events

**ni-graph-created** event is fired on the graph after the flot chart has been created the first time.

**ni-graph-updated** event is fired on the graph when Webcharts has been updated.

## ni-cartesian-graph

> A custom element that allows to graph one or multiple data sets.

An ni-cartesian-graph is a declarative way, that uses custom elements, to add a graph
to a web page. Specify your data sets using the value attribute and customize how
the data is renderended using nested custom elements such as *ni-cartesian-axis*,
*ni-cartesian-plot*, *ni-cartesian-plot-renderer* or *ni-cursor*.

Example showing how you can render a graph of prime number versus fibonacci numbers.
```html
<ni-cartesian-graph value="[[2, 3, 5, 7, 11, 13, 17, 19],
                          [0, 1, 1, 2, 3, 5, 8, 13]]">
  <ni-cartesian-axis show label="Index" axis-position="bottom"></ni-cartesian-axis>
  <ni-cartesian-axis show label="Value" axis-position="left"></ni-cartesian-axis>
  <ni-cartesian-plot show label="Prime Numbers">
    <ni-cartesian-plot-renderer line-width="2" line-stroke="red">
    </ni-cartesian-plot-renderer>
  </ni-cartesian-plot>
  <ni-cartesian-plot show label="Fibonacci Numbers">
    <ni-cartesian-plot-renderer line-width="2" line-stroke="blue">
    </ni-cartesian-plot-renderer>
  </ni-cartesian-plot>
</ni-cartesian-graph>
```


## Data types supported by the ni-cartesian-graph

Data can be passed to a graph either through writing to the element value
property or using API access (the setData() method on the element).

Here is a list of supported data types:


* **A 1D array of numerics**
```js
[a1, a2, a3]
```
Will be drawn as a single plot. It's an efficient and fast way to
pass data to the graph.

A typed array view is also allowed.


* **A 2D array of numerics**
```js
[[a1, a2, a3], [b1, b2, b3]]
```
Will be drawn as multiple plots. It's an efficient and fast way to
pass data to the graph.

An array of typed array views is also allowed.


* **An analogWaveform**
```js
var data = new NIAnalogWaveform({
    t0: new NITimestamp(),
    dt: 0.001, // 1 ms
    Y: [a1, a2, a3]
});
```
Will be drawn as a single plot. It's an
efficient and fast way to pass data to the graph


* **An array of analogWaveforms**
```js
var aw0 = new NIAnalogWaveform({
    t0: new NITimestamp(),
    dt: 0.001, // 1 ms
    Y: [a1, a2, a3]
});
var aw1 = new NIAnalogWaveform({
    t0: new NITimestamp(),
    dt: 0.001, // 1 ms
    Y: [b1, b2, b3]
});
var data = [aw0, aw1];
```
Will be drawn as multiple plots. It's an efficient and fast way to
pass data to the graph.


* **An object (cluster) of arrays**
```js
var data = {a: [a1, a2, a3], b: [b1, b2, b3]};
```
 Will be drawn as a single XY plot,
the first array will hold the X coordinates and the second the Y
coordinates. This format is only recomended when you want to display
XY data, it's not particulary efficient or fast.


* **An array of objects (clusters) of arrays**
```js
var data = [
    {a: [a1, a2, a3], b: [b1, b2, b3]},
    {c: [c1, c2, c3], d: [d1, d2, d3]}
];
```
Will be drawn as multiple XY plots, the first array in the object
(cluster) will hold the X coordinates and the second the Y coordinates.
This format is only recomended when you want to display XY data,
it's not particulary efficient or fast.


* **An array of objects (clusters)**
```js
var data = [
    {x: a1, y: b1},
    {x: a2, y: b2},
    {x: a3, y: b3},
];
```
Will be drawn as a single XY plot, the first element in the cluster
will hold the X coordinates and the second the Y coordinates. This
format is only recomended when you want to display XY data, it's not
particulary efficient or fast.


* **An array of array of objects (clusters)**
```js
var data = [
    [{x: a1, y: b1}, {x: a2, y: b2}, {x: a3, y: b3}],
    [{x: c1, y: d1}, {x: c2, y: d2}, {x: c3, y: d3}]
];
```
Will be drawn as multiple XY plots, the first element in the cluster
will hold the X coordinates and the second the Y coordinates. This
format is only recomended when you want to display XY data, it's not
particulary efficient or fast.


* **An array of a cluster of an array of points (clusters)**
```js
var data = [
    { [ {x: a1, y: b1}, {x: a2, y: b2}, {x: a3, y: b3} ] },
    { [ {x: c1, y: d1}, {x: c2, y: d2}, {x: c3, y: d3} ] }
];
```
Will be drawn as multiple XY plots. Each element in the array is
a cluster that holds on to an array of points (i.e. a cluster of
two numeric values). This format is only recomended when you want
to display XY data, it's not particulary efficient or fast.


* **An 1D array of Complex Numbers**
```js
var a1 = new NIComplex(1, 2);
var a2 = new NIComplex(2, 3);
var a3 = new NIComplex(3, 4);
var data = [a1, a2, a3]; // array of complex numbers
var data2 = ['1+2i', '2+3i', '3+4i']; // also accepted as input
```
Will be drawn as a single plot.


* **A 2D array of Complex Numbers**
```js
var a1 = new NIComplex(1, 2);
var a2 = new NIComplex(2, 3);
var b1 = new NIComplex(3, 4);
var b2 = new NIComplex(4, 5);
var data = [[a1, a2], [b1, b2]]; // arrays of complex numbers
var data2 = [['1+2i', '2+3i'], ['3+4i', '4+5i']]; // also accepted as input
```
Will be drawn as multiple plots.




## ni-chart

> A custom element that allows to chart one or multiple data sets.

The difference between a graph and a chart is that a graph plots directly the data
sets provided but the chart uses a buffer of historical values, called a
HistoryBuffer for data handling.

As you append new values to the charts' History buffer the chart is updated as
needed, on the next browser animation frame.

Example:

```html
<ni-chart id="chart1" buffer-size="1024">
    <ni-cartesian-axis show label="Time" axis-position="bottom"></ni-cartesian-axis>
    <ni-cartesian-axis show label="Amplitude" axis-position="left"></ni-cartesian-axis>
    <ni-cartesian-plot show label="Plot 1">
      <ni-cartesian-plot-renderer line-width="2" line-stroke="#a84716"></ni-cartesian-plot-renderer>
    </ni-cartesian-plot>
</ni-chart>

<script>
  var historyBuffer = $('#chart1').getHistoryBuffer();

  // write 1000 values into the chart's history buffer
  for (var i = 0; i < 10; i++) {
    historyBuffer.push(Math.sin(i));
    i += 0.01;
  }
</script>
```

The HistoryBuffer API is described [here](historyBuffer.md).

## Data types supported by ni-chart

To add data to the chart you need to write it into the history buffer. Remember to set
the width of the history buffer to be *equal with number of plots* you are writing.

Use [history buffer's](historyBuffer.md) *setWidth* method to change the number of
data series in the chart.

Here are some examples for adding numbers and arrays of numbers to the history buffer.

* **Numbers**
```js
    var chart = document.querySelector('#myChart');
    var hb = chart.getHistoryBuffer();

    hb.push(5); // add 5 to the history buffer
```
This will render a single plot

* **multiple Numbers**
```js
    var chart = document.querySelector('#myChart');
    var hb = chart.getHistoryBuffer();
    hb.setWidth(3); // sets the width of the history buffer (number of plots)

    hb.push([5, 7, 8]); // adds 5 to first data series,
                        // 7 to the second one and 8 to the third one
```
This will render 3 plots

* **array of Numbers**
```js
    var chart = document.querySelector('#myChart');
    var hb = chart.getHistoryBuffer();

    hb.appendArray([5, 7, 8]); // add 5, 7 and 8 to first data series
```
This will render a single plot.

* **arrays of arrays of Numbers**
```js
    var chart = document.querySelector('#myChart');
    var hb = chart.getHistoryBuffer();

    hb.appendArray([[5, 7], [8, 9]]); // add 5 and 7 to first data series
                                     // and 8 and 9 to the second one
```
This will render two plots.

* **analogWaveform**
```js
    var chart = document.querySelector('#myChart');
    var hb = chart.getHistoryBuffer();
    hb.setType('analogWaveform'); // or 'numeric' to switch the history buffer back to numeric data types

    var aw = new NIAnalogWaveform({
        t0: new NITimestamp(),
        dt:0.001,
        Y: [1, 2, 3, 5]
    });

    hb.push(aw); // push the analog waveform into the buffer
```
This will render one plot of analog waveforms.

* ** array of analogWaveforms**
```js
    var chart = document.querySelector('#myChart');
    var hb = chart.getHistoryBuffer();
    hb.setType('analogWaveform'); // or 'numeric' to switch the history buffer back to numeric data types
    hb.setWidth(2);

    var aw = new NIAnalogWaveform({
        t0: new NITimestamp(),
        dt:0.001,
        Y: [1, 2, 3, 5]
    });

    var aw1 = new NIAnalogWaveform({
        t0: new NITimestamp(),
        dt:0.001,
        Y: [6, 7, 8, 9]
    });

    hb.push([aw, aw1]); // push the analog waveforms into the buffer
```

This will render two plots af analog waveforms.


### ni-chart properties

**buffer-size** (number) - the size of the chart history buffer

**value** - the default value to initialize the history buffer with

       Supported formats:
       * empty string for no default value
       * 1D array for single plot charts: [1, 2, 3]
       * 2D arrays for multiple plot charts: [[1, 11], [2, 12], [3, 13]]
       * serialized history buffer: { valueType: 'HistoryBuffer', data: [[1, 2, 3], [11, 12, 13]] }

       It is possible to specify special values like NaN or Infinity as strings: [1, 2, 'NaN', 'Infinity']



### ni-chart methods


## ni-intensity-graph

> A custom element that allows to plot a 2D data set as a gradient map.

An ni-intensity-graph is a declarative way, that uses custom elements, to add an
intensity graph to a web page. Specify your data sets using the value attribute
and customize how the data is renderended using nested custom elements such as
*ni-cartesian-axis*, *ni-color-scale*.

Example:
```html
<ni-intensity-graph id="graph1" value="[[0.1, 0.2], [0.3, 0.4], [0.5, 0.6]]">
    <ni-cartesian-axis label="X" show show-label axis-position="bottom" grid-lines></ni-cartesian-axis>
    <ni-cartesian-axis label="Y" show show-label axis-position="left" grid-lines></ni-cartesian-axis>
    <ni-color-scale label="Color Scale" show show-label
        markers='[{"value":0,"color":"rgba(0,0,0,1)"},{"value":50,"color":"rgba(0,0,255,1)"},{"value":100,"color":"rgba(255,255,255,1)"}]'
        high-color="rgba(255,255,255,1)" low-color="rgba(0,0,0,1)">
    </ni-color-scale>
</ni-intensity-graph>
```

### ni-intensity-graph properties

**enable-hover** (boolean) - if true, when the mouse hovers over a value or when a
               value is tapped, a tooltip with the value will be shown

**enable-click** (boolean) - if true, when there is a click or tap near or on a value,
               a plotclick event will be fired

**hover-format** (string/function) - specifies the format of the text shown in the tooltip.


## ni-digital-graph
> A custom element that allows to graph digital waveform data.
An ni-digital-graph is a declarative way, that uses custom elements, to add a digital
graph to a web page. Specify your data sets using the value attribute and customize how
the data is renderended using nested custom elements such as *ni-digital-axis*,
*ni-digital-signal* or *ni-digital-bus*.
Example:
```html
<ni-digital-graph value="[[0, 1, 1, 0, 1], [0, 1, 1, 1, 0], [1, 1, 0, 0, 1]]">
    <ni-cartesian-axis label="Time" show show-label axis-position="bottom"></ni-cartesian-axis>
    <ni-digital-axis show axis-position="left"></ni-digital-axis>
    <ni-digital-bus label="Bus 1">
        <ni-digital-signal label="Signal 1"></ni-digital-signal>
        <ni-digital-signal label="Signal 2"></ni-digital-signal>
        <ni-digital-signal label="Signal 3"></ni-digital-signal>
    </ni-digital-bus>
</ni-digital-graph>
```


### ni-digital-graph properties

**line-width** (number) - the width of signal and bus lines.
               Use 1 for maximum performance as thicker lines can be slower,
               depending on the browser and the graphic card used. Default: 1

**scrollbars** (string) - enables horizontal or vertical scrollbars.
               Default: 'none' Accepted values: 'none', 'horizontal', 'vertical' and 'all'


## ni-digital-axis
> A custom element that adds a digital axis to a digital graph.

A digital axis displays the names of signals and buses and is capable
of expanding or collapsing individual buses.

```html
<ni-digital-graph value="[[0, 1, 1, 0, 1], [0, 1, 1, 1, 0], [1, 1, 0, 0, 1]]">
    <ni-digital-axis show axis-position="left"></ni-digital-axis>
    <ni-digital-bus label="Bus 1">
        <ni-digital-signal show label="Signal 1"></ni-digital-signal>
        <ni-digital-signal show label="Signal 2"></ni-digital-signal>
        <ni-digital-signal show label="Signal 3"></ni-digital-signal>
    </ni-digital-bus>
</ni-digital-graph>
```


### ni-digital-axis properties

**show** (boolean) - if true the axis will be visible. Default: false

**axis-position** (string) - the position of the axis. Default: 'left'
               Accepted values: 'left' and 'right'

**width** (number or string) - the width of the axis in px.
               Use 'automatic' to let the width be calculated depending on the bus and signal labels.
               Default: 'automatic'

**signal-symbol** (string) - a HTML element to use as signal symbol. Default: null

**bus-symbol-expanded** (string) - a HTML element to use as symbol for expanded buses. Default: null

**bus-symbol-collapsed** (string) - a HTML element to use as symbol for collapsed buses. Default: null


## ni-digital-signal
> A custom element that adds a signal to a digital graph.

A signal is the graphic representation of a binary data set. The signal can
optionally be nested inside a ni-digital-bus element to assign the signal
to a specific bus.

```html
<ni-digital-graph value="[[0, 1, 0, 1, 0], [0, 1, 1, 1, 0], [1, 1, 0, 0, 1]]">
    <ni-digital-signal label="Clock"></ni-digital-signal>
    <ni-digital-bus label="Bus 1">
        <ni-digital-signal label="Signal 1"></ni-digital-signal>
        <ni-digital-signal label="Signal 2"></ni-digital-signal>
    </ni-digital-bus>
</ni-digital-graph>
```


### ni-digital-signal properties

**show** (boolean) - if true the signal will be shown. Default: false

**label** (string) - the name of the signal. Default: ''


## ni-digital-bus
> A custom element that adds a bus to a digital graph.

A bus is the graphic representation of an aggregation of multiple digital signals.
The aggregated signals are determined by nested ni-digital-signal elements.

```html
<ni-digital-graph value="[[0, 1, 1, 0, 1], [0, 1, 1, 1, 0], [1, 1, 0, 0, 1]]">
    <ni-digital-bus label="Bus 1">
        <ni-digital-signal label="Signal 1"></ni-digital-signal>
        <ni-digital-signal label="Signal 2"></ni-digital-signal>
        <ni-digital-signal label="Signal 3"></ni-digital-signal>
    </ni-digital-bus>
</ni-digital-graph>
```


### ni-digital-bus properties

**label** (string) - the name of the bus. Default: ''

**line-width** (number) - the width of the line. Use 1 for maximum
               performance as thicker lines can be slower, depending on the browser and
               the graphic card used. Default: 1

**collapse** (boolean) - if true the bus will be collapsed
               meaning its signals are not displayed. Default: false

**bit-numbering** (string) - specifies whether the first signal
               in the bus is the least significant bit or most significant bit.
               Default: 'lsb'. Accepted values: 'lsb', 'msb'

**show-labels** (boolean) - if true the bus values will be drawn
               on top of each bus segment. Default: false

**label-position** (string) - specifies the position of bus labels
               between two bus value changes.
               Default: 'center'. Accepted values: 'center', 'left', 'right'

**label-format** (string) - specifies the format of bus labels.
               Default: 'binary'. Accepted values: 'binary', 'decimal', 'hex'


## ni-cartesian-axis

> Add axes to a graph.

```html
<ni-cartesian-graph>
  <ni-cartesian-axis show grid-lines show-label label="Axis 1" axis-position='bottom'>
  </ni-cartesian-axis>
  <ni-cartesian-axis show grid-lines show-label label="Axis 2" axis-position='top'>
  </ni-cartesian-axis>
  <ni-cartesian-axis show grid-lines show-label label="Axis 3" axis-position='left'>
  </ni-cartesian-axis>
</ni-cartesian-graph>
```
Multiple vertical and horizontal axes can be added to a graph and configured.

Axes can be placed to the right, left and on top or bottom of a graph, can use a
logarithmic scale if desired and can show time. A relative time format is supported
along with the absolute time format.

The minimum and maximum value of an axis can be specified. Autoscaling according
to the data ranges dispayed is also possible.


### ni-cartesian-axis properties

**axis-ref** - an axis id used to bind plots to the axes. Default: ''

**show** (boolean) - if true the axis will be visible. Default: false

**label** (string) - the name of the axis. Default: ''

**axis-position** (string) - the position of the axis. Default: 'left'
 Accepted values: 'top', 'bottom', 'left' and 'right'


**show-label** (boolean) - if true the axis label will be visible.
               Default: false

**minimum** (number) - the minimum axis value. Default: 0

**maximum** (number) - the maximum axis value. Default: 1

**auto-scale** (string) - if 'exact' or 'loose' the axis will adapt to the data
               ranges of the plots assigned to that axis. if 'none', the axis will adapt to the
               minimum and maximum options. For 'growexact' and 'growloose' the axis will adapt only when
               the new data value is bigger than the old one. Default: 'auto'

**log-scale** (boolean) - if true the axis will be a log axis. Default: false

**format** (string) - the format of the tick labels. Default: ''
Possible values: 'LVSI, 'LVTime', 'LVRelativeTime', 'LVRelativeSeconds'.
Examples:

|format|value(s)|formatted value(s)|
|------|----:|--------------:|
|_not specified_|0.1234567|0.1234567|
|LVSI|0.1234567|123.4567m|
|LVSI:0|0.1234567|123m|
|LVSI:0.###|0.1234567|123.457m|
|LVSI:G3|0.1234567|123m|
|LVSI:G3|1.234567|1.23|
|LVSI:N3|0.1234567|123.457m|
|LVTime|0|12:00:00 AM 1/1/0001|
|LVTime|300|12:05:00 AM 1/1/0001|
|LVRelativeTime|0, 300, 600|00:00:00, 00:05:00, 00:10:00|
|LVRelativeTime|300, 600, 900|00:00:00, 00:05:00, 00:10:00|
|LVRelativeSeconds:300|300|300.0|
|LVRelativeSeconds:0.###|0.12345678|0.123|
|LVRelativeSeconds:G3|0.12345678|0.123|
|LVRelativeSeconds:G3|1.2345678|1.23|
|LVRelativeSeconds:N3|1234.5678|1234.567|

The actual starting time corresponding to the 0 value for the 'LVTime' format
may vary depending on the time zone.

The first value of each axis using the 'LVRelativeTime' format is considered
the starting point for that axis and it will always be formatted as 00:00:00.

The actual number of decimals may be larger than specified for the endpoints.
When the range of an axis using the 'LVRelativeTime' format is smaller than 1
minute then the milliseconds will be displayed as well.

Other items which are displaying values, like the cursors or cursor legend,
will use this format as well.


* This value represents the total milliseconds to offset the formatted display of
* absolute time on an axis.
* The default will offset a date synonymous with 'new Date(0)' to the date:
* '12:00:00 AM 01/01/0001'


**show-tick-labels** (string) - which of the tick labels are visible.
               Possible values: 'none', 'endpoints', 'major' and 'all'. Default: 'all'.
               For 'endpoints' only the minimum and the maximum values will be visible
               at the edges of the axis. To make some intermediary values visible use
               'major', and 'all' to show all of them.

**ticks** (string) - override x-axis ticks. String must be valid JSON format.
               Example values: undefined, '1', '[1, 2, ...]', or '[[0, "tickLabel0"], [1, "tickLabel1"], ...]'
               Default: undefined

**grid-lines** (boolean) - if true the grid lines for the axis will
               be visible. Default: false

**show-ticks** (boolean) - if true the tick lines for the axis will
               be visible. A tick line is a short line which connects the tick label with
               the axis. Default: false

**show-minor-ticks** (boolean) - if true the minor tick lines for the axis
               will be visible. Minor tick lines are short lines (half of the tick line
               length) which subdivide the interval between tick lines in five intervals.
               Default: false

**tick-size** (string) - sets a fixed tick interval size for the axis
               Value can be a fixed number or an array.
               Note that for time series, the format is an array like [value, unit]
               where unit is one of "second", "minute", "hour", "day", "month" and "year".
               Default: undefined

**min-tick-size** (string) - sets the minimum tick interval size for the axis
               Value can be a fixed number or an array.
               Note that for time series, the format is an array like [value, unit]
               where unit is one of "second", "minute", "hour", "day", "month" and "year".
               Default: undefined

**inverted** (boolean) - if true the axis will display in reverse order
               (max at bottom/left, min at top/right).
               Default: false

**label-width** (number) - sets the axis width as if the labels were
               this fixed width. For x axes this will ensure a padding size equal to half
               this value on the left and right sides of the graph to ensure space for the
               first and last label but will not affect y padding.
               Default: undefined

**pan** (string) - what type of panning is allowed for the axis.
               Possible values: 'none', 'axis', 'plot', 'all'. Default: 'all'.
               For 'axis' panning can be done only when the mouse/touch is over the axis,
               while for 'plot', the panning is done only if the entire plot is panned.

**reserve-space** (boolean) - if true the axis will reserve label-width
               space even if the axis is not shown.
               Default: false

**zoom** (string) - what type of panning is allowed for the axis.
               Possible values: 'none', 'axis', 'plot', 'all'. Default: 'all'.
               For 'axis' zooming can be done only when the mouse/touch is over the axis,
               while for 'plot', the zooming is done only if the entire plot is zommed.


## ni-color-scale

> A color scale added to an intensity graph is used to configure the color
gradients.

```html
<ni-intensity-graph>
  <ni-cartesian-axis ...></ni-cartesian-axis>
  <ni-cartesian-axis ...></ni-cartesian-axis>
  <ni-color-scale show show-label label="Color Scale" low-color="blue" high-color="red">
  </ni-color-scale>
</ni-intensity-graph>
```

A single color scale can be added to an intensity graph to visualize and control
the maping from values to the color gradient displayed.

Color scales are only supported by intensity graphs, when added to other types of
webcharts they do nothing.

The color gradients are described by an array of stops, for example this:

```js
[
{"value":0,"color":"rgba(0,0,0,1)"},
{"value":0.5,"color":"rgba(0,0,255,1)"},
{"value":1,"color":"rgba(255,255,255,1)"}
]
```

describes a color gradient going from black at the stop with the value 0, to
blue at the stop with the value 0.5 and then to white at the stop with the value
1.


### ni-color-scale properties

**label** (string) - the name of the color scale. Default: ''

**show** (boolean) - if true the color scale will be visible. Default: false

**show-label** (boolean) - if true the color scale label will be visible. Default: false

**axis-position** (string) - the position of the color scale. Default: 'right'.
               Accepted values: *'left'*, *'right'*

**show-tick-labels** (string) - which of the tick labels are visible.
               Possible values: 'none', 'endpoints', 'major' and 'all'. Default: 'all'.
               For 'endpoints' only the minimum and the maximum values will be visible
               at the edges of the axis. To make some intermediary values visible use
               'major', and 'all' to show all of them.

**high-color** (string) - the color assigned to the values bigger than
               the color scale maximum. Default: '#ffffff'.
               Accepted values: a CSS color string

**low-color** (string) - the color assigned to the values smaller than
               the color scale minimum. Default: '#000000'.
               Accepted values: a CSS color string

**markers** (string) - the color gradient used to map values to colorScaleAxis.

               Default: ''.
               TODO: document the gradient format

**auto-scale** (string) - if 'exact' the color gradient will strech to adapt to the
               range of the data set provided. Default: 'auto'

**minimum** (number) - the minimum axis value. Default: 0

**maximum** (number) - the maximum axis value. Default: 1


## ni-cartesian-plot

> A custom element that, along with a nested ni-cartesian-plot-renderer describes
the way a data plot is rendered.

A plot is the graphic representation of a data set. A graph or chart can show
simultaneously multiple data sets (plots). To customize how a data set, or plot
is rendered use ni-cartesian-plot elements nested inside a graph or chart
element.

```html
<ni-cartesian-graph>
  <ni-cartesian-axis axis-ref="axis1ref" show axis-position="bottom"></ni-cartesian-axis>
  <ni-cartesian-axis axis-ref="axis2ref" show axis-position="left"></ni-cartesian-axis>
  <ni-cartesian-plot show label="plot 1" xaxis="axis1ref" yaxis="axis2ref">
    add an ni-cartesian-plot-renderer here
  </ni-cartesian-plot>
</ni-cartesian-graph>
```


### ni-cartesian-plot properties

**show** (boolean) - if true the plot will be shown. Default: false

**label** (string) - the name of the plot. Default: ''

**xaxis** (string) - the X axis that this plot will use. Default: ''.
               Accepted values: an axis-ref. In case an xaxis with the specified axis-ref
               is not found the first xaxis is used

**yaxis** (string) - the Y axis that this plot will use. Default: ''.
               Accepted values: an axis-ref. In case an yaxis with the specified axis-ref
               is not found the first yaxis is used

**enable-hover** (boolean) - if true, when the mouse hovers close to a point on the plot
               or a tap on a point is done, a tooltip with the point value will be shown

**enable-click** (boolean) - if true, when there is a click or tap near or on a point,
               a plotclick event will be fired

**hover-format** (string/function) - specifies the format of the text shown in the tooltip.


## ni-cartesian-plot-renderer

> Describes the way a plot is rendered

The four ways to render a plot are the line renderer, the area fill renderer,
the points renderer and the bar renderer. You can combine these in any way you
like for a single plot and different plots can have different renderers.

The fastest one is the line renderer and this is the recomended one for large
data sets and if you care about performance.

The render can be further customized by using the renderer properties.


### ni-cartesian-plot-renderer properties

**line-stroke** (string) - the color of the line drawn. Default: 'auto'.
               Accepted values: a CSS color string, an empty string or 'auto'. If an empty string
               is passed the line rendering for the plot is disabled. If 'auto' is passed a color
               will be automatically assigned to the plot.


**line-width** (number) - the width of the line. Use 1 for maximum
               performance as thicker lines can be slower, depending on the browser and
               the graphic card used. Default: 1

**line-style** (string) - the style of the line drawn. Default: 'solid'.
               Accepted values: *'solid'*, *'dot'*, *'mediumdash'*, *'dashdot'* and *'largedash'*


**area-fill** (string) - the color of the area fill drawn under/over
               the line. Default: ''.
               Accepted values: a CSS color string, an empty string or auto. If an empty string
               is passed the area fill for the plot is disabled. If 'auto' is passed a color
               will be automatically assigned to the fill.


**area-base-line** (string) - determines the direction the area fill.
Default: 'zero'. Accepted values:

* 'zero': fill the line towards 0
* '-infinity': fill the line towards -Infinity.
* 'infinity': fill the line towards Infinity.


**bar-fill** (string) - the color of the bars drawn. Default: ''.
               Accepted values: a CSS color string, an empty string or auto. If an empty
               string is passed the bars rendering for the plot is disabled. If 'auto'
               is passed a color will be automatically assigned to the bars.


**bar-width** (number) - the width of the bars drawn. Default: 0.75.
               This value is relative to the minimum distance between points for the plot.


**absolute-bar-width** (boolean) - if true, the value of barWidth is considered
               to be in units of the x-axis. Default: false.


**orientation** (string) - if displaying bars, this decides the orientation of the bars
               Accepted values: 'vertical' and 'horizontal'. Default: 'vertical'.
               if 'horizontal' x and y data values must be transposed.


**bar-base-line** (string) - determines the direction the bar fill.
Default: 'zero'. Accepted values:

* 'zero': fill the line towards 0
* '-infinity': fill the line towards -Infinity.
* 'infinity': fill the line towards Infinity.


**point-color** (string) - the color of the points drawn. Default: ''.
               Accepted values: a CSS color string, an empty string or auto. If an empty
               string is passed the points rendering for the plot is disabled. If 'auto'
               is passed a color will be automatically assigned to the points.


**point-size** (number) - the size of the points drawn. Default: 5.

**point-shape** (string) - determines the shape of the points.
               Default: 'ellipse'. Accepted values: 'ellipse', 'square', 'diamond',
               'triangle', 'cross'


**fill-points** (boolean) - if true, plot points will be filled with the point color.

**fill-opacity** (number) - determines the fill opacity for bars, points, and area plots. Default: 0.6.

**bar-line-width** (number) - determines the thickness of a bar's border in pixels. Default: 2.

## ni-cursor

> Add cursors to a graph.

Sometimes you want to take a measurement off a graph you have made. Cursors allow
you to do that.

```html
<ni-cartesian-graph>
  <ni-cartesian-axis show grid-lines show-label label="Axis 1" axis-position='bottom'>
  </ni-cartesian-axis>
  <ni-cartesian-axis show grid-lines show-label label="Axis 2" axis-position='top'>
  </ni-cartesian-axis>
  <ni-cursor label="Cursor 1" show show-label snap-to-plot="0"></ni-cursor>
</ni-cartesian-graph>
```
Multiple cursors can be added to a graph and configured.


### ni-cursor properties

**show** (boolean) - if true the cursor will be visible. Default: false

**label** (string) - the label of the cursor. Default: ''

**show-label** (boolean) - if true the cursor label will be visible.
               Default: false

**target-shape** (string) - the shape of cursor. Default: 'ellipse'.
Accepted values: 'ellipse', 'square', 'diamond', 'triangle', 'cross'

**target-size** (number) - the size of the cursor manipulator symbol, in pixels.
               Default: 8.

**crosshair-style** (string) - the shape of crosshair. Default: 'both'.
Accepted values: 'both', 'vertical', 'horizontal', 'none'

**show-value** (boolean) - if true the coordinates of the cursor will
               be visible. Default: false

**snap-to-plot** (number) - the number of the plot or -1 to snap to any plot.
               Default: undefined (no snapping)

**interpolate** (boolean) - if true the cursor will do a linear interpolation
               between points when modiffying position. Default: false


**xaxis** (string) - the X axis that this cursor will use when snap-to-plot is undefined.
               Default: ''. Accepted values: an axis-ref. In case an xaxis with the specified axis-ref
               is not found the first xaxis is used

**yaxis** (string) - the Y axis that this cursor will use when snap-to-plot is undefined.
               Default: ''. Accepted values: an axis-ref. In case an yaxis with the specified axis-ref
               is not found the first yaxis is used

**x** (number) - the x target coordinate of the cursor. This coordinate
               is relative to the visible area of the graph, in the interval 0 to 1.
               0 is on the left edge and 1 on the one to the right.
               Default: 0.5

**y** (number) - the y target coordinate of the cursor. This coordinate
               is relative to the visible area of the graph, in the interval 0 to 1.
               0 is on the bottom edge and 1 on the top one.
               Default: 0.5

**x-Position** (number) - the xPosition target coordinate of the cursor. This coordinate
               is based on graphs xAxis scale.

**y-Position** (number) - the yPosition target coordinate of the cursor. This coordinate
               is based on the graphs yAxis scale.

**font-family** (string) - the font used for cursor label and values.
               Default: 'sans-serif'

**font-size** (string) - the font size used for cursor label and values.
               Default: '10px'

**font-style** (string) - the font style used for cursor label and values.
               Default: ''

**show-thumbs** (string) - the visibility of thumbs and axis on which they appear.
               Default: 'none'. Accepted values: 'r', 'l', 'b', 't', 'none'

**dashes** (array) - an array of numbers to specify alternating lengths of lines and gaps.
               Default: [] (solid line cursor).

**thumb-label** (string) - the text to use on the thumb for the cursor.
               Default: '' (no label).

**thumb-offset** (number) - shifts the thumb position relative to the graph.
               A larger number shifts the thumb further from the graph
               Default: '0'

**thumb-color** (color) - specify a color to fill the cursor thumbs.
               Default: undefined (use style of thumbIcon class)

TODO: clean up and document the events generated by cursors


## ni-axis-handle

> Add axis handles to a graph.

The user can drag an axis handle to offset the associated axis.

```html
<ni-cartesian-graph>
<ni-cartesian-axis show grid-lines show-label label="Axis 1" axis-position='bottom'>
</ni-cartesian-axis>
<ni-cartesian-axis show grid-lines show-label label="Axis 2" axis-position='top'>
</ni-cartesian-axis>
<ni-axis-handle show orientation="horizontal" label="h1"></ni-axis-handle>
<ni-axis-handle show orientation="vertical" label="h2"></ni-axis-handle>
</ni-cartesian-graph>
```
Multiple axis handles can be added to a graph and configured. One axis handle per axis is recommended.


### ni-axis-handle properties

**position** (number) - the value is relative to the visible area of the graph. Default: 0.5

**absolutePosition** (number) - the axis handle position defined by the value on the axis. It has higher priority than position. Default value: undefined.

**show** (boolean) - if true the axis handle will be visible. Default: false

**orientation** (string) - the orientation of the axis handle. Default: 'vertical'

**location** (string) - the location of the axis handle. Default: 'far'

**axisIndex** (number) - the axis index on the requested orientation. Default: 0.
                This property is deprecated. Use axis instead. ni-webcharts may remove this property in the future.

**label** (string) - the label of the axis handle. Default: ''

**fill** (color) - is the color of the axis handle thumb. Default value: undefined (use fill color from CSS class .thumbIcon).

**axis** (string) - the axis that this handle targets to. Default: ''.
                 Accepted values: an axis-ref. See the [axis-ref property](https://github.com/ni-kismet/webcharts/blob/master/docs/API.md#ni-cartesian-axis-properties) in ni-cartesian-axis. In case an axis with the specified axis-ref
                is not found the first axis is used


## ni-parking-lot

> Add the parking lot to the graph.

With the parking lot a cursor/axis handle can be dragged fully off the edge of the graph.
Once the handle is off the edge of the graph, it will show up in the respective parking lot corner.

```html
<ni-cartesian-graph>
<ni-cartesian-axis show grid-lines show-label label="Axis 1" axis-position='bottom'>
</ni-cartesian-axis>
<ni-cartesian-axis show grid-lines show-label label="Axis 2" axis-position='top'>
</ni-cartesian-axis>
<ni-axis-handle show orientation="horizontal" label="AH"></ni-axis-handle>
<ni-cursor label="Cursor 1" show show-label snap-to-plot="0"></ni-cursor>
<ni-parking-lot></ni-parking-lot>
</ni-cartesian-graph>
```
A single parking lot can be added to the graph.


### ni-parking-lot properties

**offset** (number) - the distance between the graph area and the parking lot Default: 35


# TypedWebCharts Plugin

> Allows setting data types to WebCharts.

The information of the data type will be used to optimize setting
or pushing new data to the WebCharts.

Example:
```html
<ni-cartesian-graph value="[[1, 2, 3, 4, 5]]" ni-type="{&quot;name&quot;:&quot;Array&quot;,&quot;rank&quot;:1,&quot;subtype&quot;:{&quot;name&quot;:&quot;Double&quot;}}">
</ni-cartesian-graph>
```



## ni-cartesian-graph module

**ni-type** - the data type of the values being written to the
               value property or setData() method

## ni-cartesian-graph module

**ni-type** - the data type of the values being pushed to the history buffer.

## ni-intensity-graph module

**ni-type** - for now this is not used



# TypedHistoryBuffer

A TypedHistoryBuffer is a HistoryBuffer with information about the data type
of the values being pushed to it.

The data type is used exclusively for optimizing the push of new data into the buffer operation.

Example:
```javascript
var hb1 = new TypedHistoryBuffer(1024);

hb1.setNIType(NITypes.DOUBLE.makeArray());

hb1.pushTypedData([1, 2]);
hb1.pushTypedData([3, 4, 5]);

console.log(hb1.toArray()); //[1, 2, 3, 4, 5]
```

TypedHistoryBuffer with width = 2:
```javascript
var hb1 = new TypedHistoryBuffer(1024, 2);

hb1.setNIType(NITypes.DOUBLE.makeArray(2));

hb1.pushTypedData([[1, 11], [2, 22]]);
hb1.pushTypedData([[3, 33], [4, 44], [5, 55]]);

console.log(hb1.toArray()); //[[1, 11], [2, 22], [3, 33], [4, 44], [5, 55]]
```



## TypedHistoryBuffer methods

**TypedHistoryBuffer(capacity, width)** - the History buffer constructor creates
   a new history buffer with the specified capacity (default: 1024) and width (default: 1).
   The data type is set to unkown.

**setNItype(niType)** - sets the data type of the values being pushed.
       Supported data types:
       * any numeric type
       * one dimension numeric arrays
       * two dimensions numeric arrays
       * clusters of numerics
       * array of clusters of numerics


**pushTypedData(value)** - pushes a new value to the history buffer
